# -*- coding: utf-8 -*-
"""
    __init__

    Collect all tests here

"""
import unittest

from .test_banner import TestBanner, TestGetHtml
from .test_cms import TestCMS
from .test_menuitem import TestMenuItem


def suite():
    test_suite = unittest.TestSuite()
    test_suite.addTests([
        unittest.TestLoader().loadTestsFromTestCase(TestBanner),
        unittest.TestLoader().loadTestsFromTestCase(TestGetHtml),
        unittest.TestLoader().loadTestsFromTestCase(TestCMS),
        unittest.TestLoader().loadTestsFromTestCase(TestMenuItem),
    ])
    return test_suite



# -*- coding: utf-8 -*-
'''

    Nereid User
    user.py


'''
from trytond.pool import Pool, PoolMeta

from nereid import route, request, abort
from werkzeug.contrib.atom import AtomFeed

__all__ = ['NereidUser']
__metaclass__ = PoolMeta


class NereidUser:
    __name__ = 'nereid.user'

    def serialize(self, purpose=None):
        """
        Downstream implementation of serialize() which adds serialization for
        atom feeds.
        """
        if purpose == 'atom':
            return {
                'name': self.display_name,
                'email': self.email or None,
            }
        elif hasattr(super(NereidUser, self), 'serialize'):
            return super(NereidUser, self).serialize(purpose=purpose)

    @classmethod
    @route('/article-author/<int:id>.atom')
    def atom_feed(cls, id):
        """
        Returns the atom feed for all articles published under a certain author
        """
        Article = Pool().get('nereid.cms.article')

        try:
            articles = Article.search([
                ('author', '=', id),
                ('state', '=', 'published'),
            ])
        except:
            abort(404)

        feed = AtomFeed(
            "Articles by Author %s" % cls(id).display_name,
            feed_url=request.url, url=request.host_url
        )
        for article in articles:
            feed.add(**article.serialize(purpose='atom'))

        return feed.get_response()



# -*- coding: utf-8 -*-
'''

    nereid_cms test_banner


'''
import unittest

from lxml import objectify
import trytond.tests.test_tryton
from trytond.tests.test_tryton import POOL, USER, ModuleTestCase, \
    with_transaction
from nereid.testing import NereidTestCase


class TestBanner(NereidTestCase, ModuleTestCase):
    """Test Banners"""
    module = 'nereid_cms'

    def setUp(self):
        trytond.tests.test_tryton.install_module('nereid_cms')

        self.Currency = POOL.get('currency.currency')
        self.Site = POOL.get('nereid.website')
        self.BannerCategory = POOL.get('nereid.cms.banner.category')
        self.Banner = POOL.get('nereid.cms.banner')
        self.Folder = POOL.get('nereid.static.folder')
        self.File = POOL.get('nereid.static.file')
        self.Company = POOL.get('company.company')
        self.NereidUser = POOL.get('nereid.user')
        self.Language = POOL.get('ir.lang')
        self.NereidWebsite = POOL.get('nereid.website')
        self.Party = POOL.get('party.party')
        self.Locale = POOL.get('nereid.website.locale')

        self.templates = {
            'home.jinja':
            '''
                {% for banner in get_banner_category("test-banners").banners %}
                    {{ banner.get_html()|safe }}
                {% endfor %}
            ''',
        }

    def get_template_source(self, name):
        """
        Return templates
        """
        return self.templates.get(name)

    def setup_defaults(self):
        """
        Setup the defaults
        """
        usd, = self.Currency.create([{
            'name': 'US Dollar',
            'code': 'USD',
            'symbol': '$',
        }])
        company_party, = self.Party.create([{
            'name': 'Openlabs'
        }])
        company, = self.Company.create([{
            'party': company_party,
            'currency': usd
        }])
        guest_party, = self.Party.create([{
            'name': 'Guest User',
        }])
        guest_user, = self.NereidUser.create([{
            'party': guest_party,
            'display_name': 'Guest User',
            'email': 'guest@openlabs.co.in',
            'password': 'password',
            'company': company,
        }])
        registered_party, = self.Party.create([{
            'name': 'Registered User'
        }])
        self.registered_user, = self.NereidUser.create([{
            'party': registered_party,
            'display_name': 'Registered User',
            'email': 'email@example.com',
            'password': 'password',
            'company': company,
        }])

        # Create website
        en_us, = self.Language.search([('code', '=', 'en_US')])
        self.locale_en_us, = self.Locale.create([{
            'code': 'en_US',
            'language': en_us.id,
            'currency': usd.id
        }])
        return self.NereidWebsite.create([{
            'name': 'localhost',
            'company': company,
            'application_user': USER,
            'default_locale': self.locale_en_us.id,
            'currencies': [('add', [usd.id])],
        }])[0]

    @with_transaction()
    def test_0010_banner_categ(self):
        """All banners in published state.

        The banners attribute of the banner category returns all the banners
        irrespective of the status. The attribute published_banners must only
        return the active banners.

        This test creates four banner of which two are later archived, and the
        test ensures that there are only two published banners
        """
        banner_categ1, = self.BannerCategory.create([{
            'name': 'CAT-A'
        }])
        banner_categ2, = self.BannerCategory.create([{
            'name': 'CAT-B'
        }])

        self.Banner.create([{
            'name': 'CAT-A1',
            'category': banner_categ1,
            'type': 'custom_code',
            'custom_code': 'Custom code A1',
            'state': 'archived'
        }])
        self.Banner.create([{
            'name': 'CAT-A2',
            'category': banner_categ1,
            'type': 'custom_code',
            'custom_code': 'Custom code A2',
            'state': 'published'
        }])
        self.Banner.create([{
            'name': 'CAT-B1',
            'category': banner_categ2,
            'type': 'custom_code',
            'custom_code': 'Custom code B1',
            'state': 'archived'
        }])
        self.Banner.create([{
            'name': 'CAT-B2',
            'category': banner_categ2,
            'type': 'custom_code',
            'custom_code': 'Custom code B2',
            'state': 'published'
        }])

        self.assertEqual(len(banner_categ1.banners), 2)
        self.assertEqual(len(banner_categ1.banners), 2)
        self.assertEqual(len(banner_categ1.published_banners), 1)
        self.assertEqual(len(banner_categ1.published_banners), 1)

    @with_transaction()
    def test_0020_banner_image(self):
        """
        Test the image type banner created using static files
        """
        site = self.setup_defaults()

        category, = self.BannerCategory.create([{
            'name': 'test-banners',
            'website': site
        }])
        folder, = self.Folder.create([{
            'description': 'image',
            'name': 'image'
        }])
        file, = self.File.create([{
            'name': 'logo.png',
            'folder': folder,
        }])
        self.Banner.create([{
            'name': 'Test Image Banner',
            'category': category,
            'type': 'media',
            'file': file,
            'state': 'published'
        }])

        app = self.get_app()
        with app.test_client() as c:
            response = c.get('/')
            html = objectify.fromstring(response.data)
            self.assertEqual(
                html.find('img').get('src'),
                '/static-file/image/logo.png'
            )

    @with_transaction()
    def test_0040_custom_code(self):
        """
        Test the custom code
        """
        site = self.setup_defaults()

        category, = self.BannerCategory.create([{
            'name': 'test-banners',
            'website': site
        }])
        self.Banner.create([{
            'name': 'Test custom code Banner',
            'category': category,
            'type': 'custom_code',
            'custom_code': 'some ultra complex custom code',
            'state': 'published'
        }])

        app = self.get_app()
        with app.test_client() as c:
            response = c.get('/')
            self.assertTrue(
                'some ultra complex custom code' in response.data,
            )


class TestGetHtml(NereidTestCase, ModuleTestCase):
    """Test Get Html for Banners
    """
    module = 'nereid_cms'

    def setUp(self):
        trytond.tests.test_tryton.install_module('nereid_cms')

        self.Currency = POOL.get('currency.currency')
        self.Banner = POOL.get('nereid.cms.banner')
        self.BannerCategory = POOL.get('nereid.cms.banner.category')
        self.File = POOL.get('nereid.static.file')
        self.Folder = POOL.get('nereid.static.folder')
        self.Company = POOL.get('company.company')
        self.NereidUser = POOL.get('nereid.user')
        self.Language = POOL.get('ir.lang')
        self.NereidWebsite = POOL.get('nereid.website')
        self.Party = POOL.get('party.party')
        self.Locale = POOL.get('nereid.website.locale')

        self.templates = {
            'home.jinja':
            '''
                {% for b in get_banner_category('Category A').banners -%}
                {{ b.get_html()|safe }}
                {%- endfor %}
            ''',
            'article-category.jinja': '{{ articles|length }}',
            'article.jinja': '{{ article.content }}',
        }

    def get_template_source(self, name):
        """
        Return templates
        """
        return self.templates.get(name)

    def setup_defaults(self):
        """
        Setup the defaults
        """
        usd, = self.Currency.create([{
            'name': 'US Dollar',
            'code': 'USD',
            'symbol': '$',
        }])
        company_party, = self.Party.create([{
            'name': 'Openlabs'
        }])
        company, = self.Company.create([{
            'party': company_party,
            'currency': usd
        }])
        guest_party, = self.Party.create([{
            'name': 'Guest User',
        }])
        guest_user, = self.NereidUser.create([{
            'party': guest_party,
            'display_name': 'Guest User',
            'email': 'guest@openlabs.co.in',
            'password': 'password',
            'company': company,
        }])
        registered_party, = self.Party.create([{
            'name': 'Registered User'
        }])
        self.registered_user, = self.NereidUser.create([{
            'party': registered_party,
            'display_name': 'Registered User',
            'email': 'email@example.com',
            'password': 'password',
            'company': company,
        }])

        # Create website
        en_us, = self.Language.search([('code', '=', 'en_US')])
        self.locale_en_us, = self.Locale.create([{
            'code': 'en_US',
            'language': en_us.id,
            'currency': usd.id
        }])
        return self.NereidWebsite.create([{
            'name': 'localhost',
            'company': company,
            'application_user': USER,
            'default_locale': self.locale_en_us.id,
            'currencies': [('add', [usd.id])],
        }])[0]

    @with_transaction()
    def test_0010_get_html(self):
        """
        Get Html for banners with type `image`.
        """
        site = self.setup_defaults()

        banner_category, = self.BannerCategory.create([{
            'name': 'Category A',
            'website': site,
        }])

        image, = self.Folder.create([{
            'description': 'image',
            'name': 'image'
        }])
        file, = self.File.create([{
            'name': 'logo.png',
            'folder': image,
        }])
        self.Banner.create([{
            'name': 'Test Banner1',
            'category': banner_category,
            'type': 'media',
            'file': file,
            'state': 'published'
        }])

        app = self.get_app()
        with app.test_client() as c:
            rv = c.get('/')
            html = objectify.fromstring(rv.data)
            self.assertEqual(
                html.find('img').get('src'),
                '/static-file/image/logo.png'
            )

    @with_transaction()
    def test_0030_get_html(self):
        """
        Get Html for banners with type `custom_code`.
        """
        self.setup_defaults()

        banner_category, = self.BannerCategory.create([{
            'name': 'Category C'
        }])

        banner, = self.Banner.create([{
            'name': 'Test Banner3',
            'category': banner_category,
            'type': 'custom_code',
            'custom_code': 'Custom code for Test Banner3',
            'state': 'published'
        }])
        rv = banner.get_html()
        self.assertEqual(rv, banner.custom_code)


def suite():
    "Nereid CMS Banners test suite"
    test_suite = unittest.TestSuite()
    test_suite.addTests([
        unittest.TestLoader().loadTestsFromTestCase(TestBanner),
        unittest.TestLoader().loadTestsFromTestCase(TestGetHtml),
    ])
    return test_suite


if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



# -*- coding: utf-8 -*-
'''

    nereid_cms


'''

from trytond.pool import Pool
from .cms import (
    MenuItem, BannerCategory, Banner, ArticleCategory,
    Article, ArticleAttribute, Website, NereidStaticFile,
    ArticleCategoryRelation,
)
from user import NereidUser


def register():
    """
    Register classes
    """
    Pool.register(
        MenuItem,
        BannerCategory,
        Banner,
        ArticleCategory,
        Article,
        ArticleAttribute,
        NereidStaticFile,
        Website,
        ArticleCategoryRelation,
        NereidUser,
        module='nereid_cms', type_='model'
    )



# -*- coding: utf-8 -*-
'''

    nereid_cms test_cms


'''
import unittest

import trytond.tests.test_tryton
from trytond.tests.test_tryton import (
    POOL, USER, with_transaction
)
from nereid.testing import NereidTestCase


class TestCMS(NereidTestCase):
    """Test CMS"""

    def setUp(self):
        trytond.tests.test_tryton.install_module('nereid_cms')

        self.Currency = POOL.get('currency.currency')
        self.ArticleCategory = POOL.get('nereid.cms.article.category')
        self.Article = POOL.get('nereid.cms.article')
        self.Folder = POOL.get('nereid.static.folder')
        self.File = POOL.get('nereid.static.file')
        self.Company = POOL.get('company.company')
        self.NereidUser = POOL.get('nereid.user')
        self.Language = POOL.get('ir.lang')
        self.Website = POOL.get('nereid.website')
        self.ArticleAttribute = POOL.get('nereid.cms.article.attribute')
        self.Party = POOL.get('party.party')
        self.Locale = POOL.get('nereid.website.locale')
        self.MenuItem = POOL.get('nereid.cms.menuitem')

        self.templates = {
            'home.jinja':
            '''{% for banner in get_banner_category("test-banners").banners %}
            {{ banner.get_html(banner.id)|safe }}
            {% endfor %}
            ''',
            'article-category.jinja': '{{ articles|length }}',
            'article.jinja': '{{ article.content }}',
            'test-category.jinja':
            '''{% for article in articles %}
            {{ article.uri }}
            {% endfor %}
            ''',
        }

    def get_template_source(self, name):
        """
        Return templates
        """
        return self.templates.get(name)

    def setup_defaults(self):
        """
        Setup the defaults
        """
        usd, = self.Currency.create([{
            'name': 'US Dollar',
            'code': 'USD',
            'symbol': '$',
        }])
        company_party, = self.Party.create([{
            'name': 'Openlabs'
        }])
        company, = self.Company.create([{
            'party': company_party,
            'currency': usd
        }])
        guest_party, = self.Party.create([{
            'name': 'Guest User',
        }])
        guest_user, = self.NereidUser.create([{
            'party': guest_party,
            'display_name': 'Guest User',
            'email': 'guest@openlabs.co.in',
            'password': 'password',
            'company': company.id,
        }])

        registered_party, = self.Party.create([{
            'name': 'Registered User'
        }])
        self.registered_user, = self.NereidUser.create([{
            'party': registered_party,
            'display_name': 'Registered User',
            'email': 'email@example.com',
            'password': 'password',
            'company': company.id,
        }])

        # Create locale
        en_us, = self.Language.search([('code', '=', 'en_US')])
        self.locale_en_us, = self.Locale.create([{
            'code': 'en_US',
            'language': en_us.id,
            'currency': usd.id
        }])
        # Create website
        self.Website.create([{
            'name': 'localhost',
            'company': company.id,
            'application_user': USER,
            'default_locale': self.locale_en_us.id,
            'currencies': [('add', [usd.id])],
        }])

        # Create an article category
        self.article_categ, = self.ArticleCategory.create([{
            'title': 'Test Categ',
            'unique_name': 'test-categ',
        }])

        self.Article.create([{
            'title': 'Test Article',
            'uri': 'test-article',
            'content': 'Test Content',
            'sequence': 10,
            'categories': [('add', [self.article_categ.id])],
        }])

    @with_transaction()
    def test_0090_article_states(self):
        """All articles in published state.

        The articles attribute of the article category returns all the articles
        irrespective of the status. The attribute published_articles must only
        return the active articles.

        This test creates four articles of which two are later archived, and
        the test ensures that there are only two published articles
        """
        article_categ1, = self.ArticleCategory.create([{
            'title': 'Test Categ',
            'unique_name': 'test-categ1',
        }])
        article_categ2, = self.ArticleCategory.create([{
            'title': 'Test Categ',
            'unique_name': 'test-categ2',
        }])

        self.Article.create([{
            'title': 'Test Article',
            'uri': 'test-article1',
            'content': 'Test Content',
            'sequence': 10,
            'categories': [('add', [article_categ1.id])],
            'state': 'archived'
        }])
        self.Article.create([{
            'title': 'Test Article',
            'uri': 'test-article2',
            'content': 'Test Content',
            'sequence': 20,
            'categories': [('add', [article_categ1.id])],
            'state': 'published'
        }])
        self.Article.create([{
            'title': 'Test Article',
            'uri': 'test-article3',
            'content': 'Test Content',
            'sequence': 30,
            'categories': [('add', [article_categ2.id])],
            'state': 'archived'
        }])
        self.Article.create([{
            'title': 'Test Article',
            'uri': 'test-article4',
            'content': 'Test Content',
            'sequence': 40,
            'categories': [('add', [article_categ2.id])],
            'state': 'published'
        }])

        self.assertEqual(len(article_categ1.articles), 2)
        self.assertEqual(len(article_categ2.articles), 2)
        self.assertEqual(len(article_categ1.published_articles), 1)
        self.assertEqual(len(article_categ2.published_articles), 1)

    @with_transaction()
    def test_0010_article_category(self):
        "Successful rendering of an article_category page"
        self.setup_defaults()
        app = self.get_app()
        self.Article.create([{
            'title': 'Test Article',
            'uri': 'test-article1',
            'content': 'Test Content',
            'sequence': 10,
            'categories': [('add', [self.article_categ.id])],
            'state': 'published'
        }, {
            'title': 'Test Article',
            'uri': 'test-article2',
            'content': 'Test content',
            'sequence': 20,
            'categories': [('add', [self.article_categ.id])],
            'state': 'draft'
        }])
        with app.test_client() as c:
            response = c.get('/article-category/test-categ/')
            self.assertEqual(response.status_code, 200)
            self.assertEqual(response.data, '1')
            self.assertEqual(len(self.article_categ.published_articles), 1)
            self.assertEqual(len(self.article_categ.articles), 3)

    @with_transaction()
    def test_0020_article(self):
        "Successful rendering of an article page"
        self.setup_defaults()
        app = self.get_app()

        # Publish the article first
        article, = self.Article.search([
            ('uri', '=', 'test-article')
        ])
        self.Article.publish([article])

        with app.test_client() as c:
            response = c.get('/article/test-article')
            self.assertEqual(response.status_code, 200)
            self.assertEqual(response.data, 'Test Content')

    @with_transaction()
    def test_0030_sitemapindex(self):
        '''
        Successful index rendering
        '''
        self.setup_defaults()
        app = self.get_app(DEBUG=True)
        with app.test_client() as c:
            response = c.get('/sitemaps/article-category-index.xml')
            self.assertEqual(response.status_code, 200)

    @with_transaction()
    def test_0040_category_sitemap(self):
        '''
        Successful rendering artical catagory sitemap
        '''
        self.setup_defaults()
        app = self.get_app()
        with app.test_client() as c:
            response = c.get('/sitemaps/article-category-1.xml')
            self.assertEqual(response.status_code, 200)

    @with_transaction()
    def test_0050_article_attribute(self):
        '''
        Test creating and deleting an Article with attributes
        '''
        article_category, = self.ArticleCategory.create([{
            'title': 'Test Categ',
            'unique_name': 'test-categ',
        }])

        article1, = self.Article.create([{
            'title': 'Test Article',
            'uri': 'Test Article',
            'content': 'Test Content',
            'sequence': 10,
            'categories': [('add', [article_category.id])],
            'attributes': [
                ('create', [{
                    'name': 'google+',
                    'value': 'abc',
                }])
            ]
        }])
        # Checks an article is created with attributes
        self.assert_(article1.id)
        self.assertEqual(self.ArticleAttribute.search([], count=True), 1)
        # Checks that if an article is deleted then respective attributes
        # are also deleted.
        self.Article.delete([article1])
        self.assertEqual(self.ArticleAttribute.search([], count=True), 0)

    @with_transaction()
    def test_0055_article_content(self):
        """
        Tests that the article has been rendered properly.
        """
        article_category, = self.ArticleCategory.create([{
            'title': 'Test Categ',
            'unique_name': 'test-categ',
        }])

        article1, = self.Article.create([{
            'title': 'Test Article',
            'uri': 'Test Article',
            'content': 'Test Content',
            'content_type': 'plain',
            'sequence': 10,
            'categories': [('add', [article_category.id])],
            'attributes': [
                ('create', [{
                    'name': 'google+',
                    'value': 'abc',
                }])
            ]
        }])

        # Plain content.
        self.assertEqual(article1.__html__(), article1.content)

        # HTML content.
        article1.content = '<html><body><p>A paragraph.</p></body></html>'
        article1.content_type = 'html'

        self.assertEqual(article1.__html__(), article1.content)

        # Markdown content.
        article1.content = '**This is strong in markdown**'
        article1.content_type = 'markdown'
        article1.save()

        self.assertIn(
            '<strong>This is strong in markdown</strong>',
            article1.__html__()
        )

        article1.content = '`A blockquote`'
        article1.save()

        self.assertIn(
            '<p><code>A blockquote</code></p>',
            article1.__html__()
        )

        # RST content.
        article1.content = '*This is emphasis in rst*'
        article1.content_type = 'rst'
        article1.save()

        self.assertIn(
            '<em>This is emphasis in rst</em>',
            article1.__html__()
        )

    @with_transaction()
    def test_0060_atom_feeds(self):
        """
        Tests that the render of atom xml feeds is working correctly.
        """
        self.setup_defaults()
        app = self.get_app()

        article_categ1, = self.ArticleCategory.create([{
            'title': 'Test Categ',
            'unique_name': 'test-categ1',
        }])
        article_categ2, = self.ArticleCategory.create([{
            'title': 'Test Categ',
            'unique_name': 'test-categ2',
        }])

        self.Article.create([{
            'title': 'Test Article',
            'uri': 'test-article1',
            'content': 'Test Content',
            'sequence': 10,
            'categories': [('add', [article_categ1.id])],
            'state': 'published',
            'author': self.registered_user.id,
        }])
        self.Article.create([{
            'title': 'Test Article',
            'uri': 'test-article2',
            'content': 'Test Content',
            'sequence': 20,
            'categories': [('add', [article_categ1.id])],
            'state': 'published',
            'author': self.registered_user.id,
        }])
        self.Article.create([{
            'title': 'Test Article',
            'uri': 'test-article3',
            'content': 'Test Content',
            'sequence': 30,
            'categories': [('add', [article_categ2.id])],
            'state': 'archived',
            'author': self.registered_user.id,
        }])
        self.Article.create([{
            'title': 'Test Article',
            'uri': 'test-article4',
            'content': 'Test Content',
            'sequence': 40,
            'categories': [('add', [article_categ2.id])],
            'state': 'published',
            'author': self.registered_user.id,
        }])

        with app.test_client() as c:
            # Try rendering all articles.
            rv = c.get('/article/all.atom')
            self.assertEqual(
                rv.data.count('<entry'),
                len(self.Article.search([('state', '=', 'published')]))
            )

            rv = c.get(
                '/article-category/%s.atom' % article_categ1.unique_name
            )
            self.assertEqual(
                rv.data.count('<entry'),
                len(article_categ1.published_articles)
            )

            rv = c.get('/article-author/%d.atom' % self.registered_user.id)
            self.assertEqual(
                rv.data.count('<entry'),
                len(self.Article.search([
                    ('author', '=', self.registered_user.id),
                    ('state', '=', 'published'),
                ]))
            )

            # Try rendering for a category that does not exist.
            rv = c.get('/article-category/%d.atom' % 70)
            self.assertEqual(rv.status_code, 404)

    @with_transaction()
    def test_0065_sort_articles_by_sequence_on_article_category_page(self):
        "Sort Articles by SEquence on Articles Category Page"
        self.setup_defaults()

        self.article_categ.sort_order = 'sequence'
        self.article_categ.template = 'test-category.jinja'
        self.article_categ.save()

        article1, article2 = self.Article.create([{
            'title': 'Test Article',
            'uri': 'test-article1',
            'content': 'Test Content',
            'sequence': 30,
            'categories': [('add', [self.article_categ.id])],
            'state': 'published',
        }, {
            'title': 'Test Article',
            'uri': 'test-article2',
            'content': 'Test Content',
            'sequence': 20,
            'categories': [('add', [self.article_categ.id])],
            'state': 'published',
        }])

        app = self.get_app()
        with app.test_client() as c:
            rv = c.get('/article-category/test-categ/')
        self.assertTrue(
            rv.data.find(article1.uri) > rv.data.find(article2.uri)
        )


def suite():
    "CMS test suite"
    test_suite = unittest.TestSuite()
    test_suite.addTests(
        unittest.TestLoader().loadTestsFromTestCase(TestCMS),
    )
    return test_suite


if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



# -*- coding: utf-8 -*-
'''

    nereid_cms test_menuitem


'''
import unittest

import trytond.tests.test_tryton
from trytond.tests.test_tryton import POOL, USER, with_transaction
from nereid.testing import NereidTestCase


class TestMenuItem(NereidTestCase):
    """Test menu_item"""

    def get_template_source(self, name):
        """
        Return templates
        """
        return self.templates.get(name)

    def setUp(self):
        trytond.tests.test_tryton.install_module('nereid_cms')

        self.Article = POOL.get('nereid.cms.article')
        self.ArticleCategory = POOL.get('nereid.cms.article.category')
        self.MenuItem = POOL.get('nereid.cms.menuitem')

        self.Currency = POOL.get('currency.currency')
        self.Company = POOL.get('company.company')
        self.NereidUser = POOL.get('nereid.user')
        self.Language = POOL.get('ir.lang')
        self.Website = POOL.get('nereid.website')
        self.Party = POOL.get('party.party')
        self.Locale = POOL.get('nereid.website.locale')

    def setup_defaults(self):
        """
        Setup the defaults
        """
        usd, = self.Currency.create([{
            'name': 'US Dollar',
            'code': 'USD',
            'symbol': '$',
        }])
        company_party, = self.Party.create([{
            'name': 'Openlabs'
        }])
        company, = self.Company.create([{
            'party': company_party,
            'currency': usd
        }])
        guest_party, = self.Party.create([{
            'name': 'Guest User',
        }])
        guest_user, = self.NereidUser.create([{
            'party': guest_party,
            'display_name': 'Guest User',
            'email': 'guest@openlabs.co.in',
            'password': 'password',
            'company': company.id,
        }])

        registered_party, = self.Party.create([{
            'name': 'Registered User'
        }])
        self.registered_user, = self.NereidUser.create([{
            'party': registered_party,
            'display_name': 'Registered User',
            'email': 'email@example.com',
            'password': 'password',
            'company': company.id,
        }])

        # Create locale
        en_us, = self.Language.search([('code', '=', 'en_US')])
        self.locale_en_us, = self.Locale.create([{
            'code': 'en_US',
            'language': en_us.id,
            'currency': usd.id
        }])
        # Create website
        self.Website.create([{
            'name': 'localhost',
            'company': company.id,
            'application_user': USER,
            'default_locale': self.locale_en_us.id,
            'currencies': [('add', [usd.id])],
        }])

    @with_transaction()
    def test_0010__menuitem(self):
        """
        Test creation of menuitem
        """
        category, = self.ArticleCategory.create([{
            'title': 'blog',
            'unique_name': 'blog',
        }])

        article, = self.Article.create([{
            'uri': 'hello-world',
            'title': 'Hello World',
            'content': 'Test content',
            'sequence': 10,
            'state': 'published',
            'categories': [('add', [category.id])],
        }])
        main_view, = self.MenuItem.create([{
            'type_': 'view',
            'title': 'Test Title',
        }])
        menu1, menu2, menu3 = self.MenuItem.create([{
            'type_': 'static',
            'title': 'Test Title',
            'link': 'http://openlabs.co.in/',
            'parent': main_view
        }, {
            'type_': 'record',
            'title': 'About Us',
            'record': '%s,%s' % (article.__name__, article.id),
            'parent': main_view
        }, {
            'type_': 'record',
            'title': 'Blog',
            'record': '%s,%s' % (category.__name__, category.id),
            'parent': main_view
        }])

        self.assert_(menu1)
        self.assert_(menu2)
        self.assert_(menu3)

        self.setup_defaults()
        app = self.get_app()
        with app.test_request_context('/'):
            rv = main_view.get_menu_item(max_depth=10)
        for child in rv['children']:
            if child['type_'] == 'record' and child['record'] == category:
                self.assertEqual(len(child['children']), 1)


def suite():
    suite = unittest.TestSuite()
    suite.addTests(
        unittest.TestLoader().loadTestsFromTestCase(TestMenuItem)
    )
    return suite


if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



# -*- coding: utf-8 -*-
'''

    Nereid CMS


'''
import time
from string import Template
import pytz
from datetime import datetime

from nereid import context_processor
from nereid import (
    render_template, request, login_required, jsonify, redirect, flash,
    abort, route, current_user, current_website
)
from nereid.helpers import slugify, url_for
from nereid.contrib.pagination import Pagination
from nereid.contrib.sitemap import SitemapIndex, SitemapSection
from werkzeug.utils import secure_filename
from werkzeug.contrib.atom import AtomFeed
from nereid.ctx import has_request_context

from trytond.pyson import Eval, Not, Equal
from trytond.model import ModelSQL, ModelView, fields, Workflow, Unique
from trytond.transaction import Transaction
from trytond.pool import Pool, PoolMeta

try:
    from docutils.core import publish_parts
except ImportError:
    publish_parts = None

try:
    from markdown import markdown
except ImportError:
    markdown = None

__all__ = [
    'MenuItem', 'BannerCategory', 'Banner', 'Website',
    'ArticleCategory', 'Article', 'ArticleAttribute', 'NereidStaticFile',
    'ArticleCategoryRelation',
]
__metaclass__ = PoolMeta


class CMSMenuItemMixin(object):
    "Basic Mixin for cms menu item"

    def get_absolute_url(self, *args, **kwargs):
        """
        Return url for menu item
        """
        raise NotImplementedError(
            "Method 'get_absolute_url' is not implemented in %s" % self.__name__
        )

    def get_children(self, max_depth):
        """
        Return serialized menu_item for current menu_item children
        """
        return []

    def get_menu_item(self, max_depth):
        """
        Return huge dictionary with serialized menu item

        {
            title: <display name>,
            target: <href target>,
            link: <url>,  # if type_ is `static`
            children: [   # direct children or record children
                <menu_item children>,
                ...
            ],
            record: <instance of record>  # if type_ is `record`
        }
        """
        raise NotImplementedError(
            "Method 'get_menu_item' is not implemented in %s" % self.__name__
        )


class MenuItem(ModelSQL, ModelView, CMSMenuItemMixin):
    "Nereid CMS Menuitem"
    __name__ = 'nereid.cms.menuitem'
    _rec_name = 'title'

    type_ = fields.Selection([
        ('view', 'Parent Menu (Not link)'),
        ('static', 'Web Address'),
        ('record', 'Link to product/category/article'),
    ], 'Type', required=True, select=True)
    active = fields.Boolean('Active', select=True)
    title = fields.Char(
        'Title', required=True, select=True, translate=True, depends=['type_'],
        states={
            'required': Eval('type_') == 'static',
        }
    )
    link = fields.Char(
        'Web Address', states={
            'required': Eval('type_') == 'static',
            'invisible': Eval('type_') != 'static',
        }, depends=['type_']
    )
    target = fields.Selection([
        ('_self', 'Self'),
        ('_blank', 'Blank'),
    ], 'Target', required=True)

    parent = fields.Many2One(
        'nereid.cms.menuitem', 'Parent Menuitem', states={
            'required': Eval('type_') != 'view',
        }, depends=['type_'], select=True
    )
    child = fields.One2Many(
        'nereid.cms.menuitem', 'parent', string='Child Menu Items'
    )

    sequence = fields.Integer('Sequence', required=True, select=True)
    record = fields.Reference(
        'Select Record', selection='allowed_models', states={
            'required': Eval('type_') == 'record',
            'invisible': Eval('type_') != 'record',
        }, depends=['type_'],
    )

    @classmethod
    def allowed_models(cls):
        return [
            (None, ''),
            ('nereid.cms.article.category', 'CMS Article Category'),
            ('nereid.cms.article', 'CMS Article'),
        ]

    @staticmethod
    def default_type_():
        return 'static'

    @staticmethod
    def default_target():
        return '_self'

    @staticmethod
    def default_sequence():
        return 10

    @staticmethod
    def default_active():
        return True

    @classmethod
    def __setup__(cls):
        super(MenuItem, cls).__setup__()
        cls._error_messages.update({
            'recursion_error':
            'Error ! You can not create recursive menuitems.',
        })
        cls._order.insert(0, ('sequence', 'ASC'))

    @classmethod
    def validate(cls, menus):
        super(MenuItem, cls).validate(menus)
        cls.check_recursion(menus)

    def get_rec_name(self, name):
        def _name(menuitem):
            if menuitem.parent:
                return _name(menuitem.parent) + ' / ' + menuitem.title
            else:
                return menuitem.title
        return _name(self)

    def get_menu_item(self, max_depth):
        """
        Return huge dictionary with serialized menu item

        {
            title: <display name>,
            target: <href target>,
            link: <url>,  # if type_ is `static`
            children: [   # direct children or record children
                <menu_item children>,
                ...
            ],
            record: <instance of record>  # if type_ is `record`
        }
        """
        res = {
            'title': self.title,
            'target': self.target,
            'type_': self.type_,
        }
        if self.type_ == 'static':
            res['link'] = self.link

        if self.type_ == 'record':
            res['record'] = self.record
            res['link'] = has_request_context() and self.record.get_absolute_url()
            # Fetch record's menu item with depth 0, as childrens are not
            # needed. This allow downstream modules to add new keys to menu
            # item based on records. Like adding an image to article menu item.
            try:
                record_as_item = self.record.get_menu_item(0)
            except NotImplementedError:
                pass
            else:
                record_as_item.update(res)
                res = record_as_item

        if max_depth > 0:
            res['children'] = self.get_children(max_depth=max_depth - 1)

        if (self.type_ == 'record' and not res.get('children') and
                max_depth > 0):
            res['children'] = self.record.get_children(
                max_depth=max_depth - 1
            )
        return res

    def get_children(self, max_depth):
        """
        Return serialized menu_item for current menu_item children
        """
        children = self.search([
            ('parent', '=', self.id),
            ('active', '=', True)
        ])
        return [
            child.get_menu_item(max_depth=max_depth - 1) for child in children
        ]

    def get_absolute_url(self, *args, **kwargs):
        """
        Return url for menu item
        """
        if self.type_ == 'record':
            return self.record.get_absolute_url(*args, **kwargs)
        return self.link


class BannerCategory(ModelSQL, ModelView):
    """Collection of related Banners"""
    __name__ = 'nereid.cms.banner.category'

    name = fields.Char('Name', required=True, select=True)
    banners = fields.One2Many(
        'nereid.cms.banner', 'category', 'Banners',
        context={'published': True}
    )
    website = fields.Many2One('nereid.website', 'WebSite', select=True)
    published_banners = fields.Function(
        fields.One2Many(
            'nereid.cms.banner', 'category', 'Published Banners'
        ), 'get_published_banners'
    )

    @classmethod
    @context_processor('get_banner_category')
    def get_banner_category(cls, uri, silent=True):
        """Returns the browse record of the article category given by uri
        """
        category = cls.search([
            ('name', '=', uri),
            ('website', '=', current_website.id)
        ], limit=1)
        if not category and not silent:
            raise RuntimeError("Banner category %s not found" % uri)
        return category[0] if category else None

    def get_published_banners(self, name):
        """
        Get the published banners.
        """
        NereidBanner = Pool().get('nereid.cms.banner')
        res = []
        banners = NereidBanner.search([
            ('state', '=', 'published'),
            ('category', '=', self.id)
        ])
        for banner in NereidBanner.browse(banners):
            res.append(banner.id)
        return res


class Banner(Workflow, ModelSQL, ModelView):
    """Banner for CMS."""
    __name__ = 'nereid.cms.banner'

    name = fields.Char('Name', required=True, select=True)
    description = fields.Text('Description')
    category = fields.Many2One(
        'nereid.cms.banner.category', 'Category', required=True, select=True
    )
    sequence = fields.Integer('Sequence', select=True)

    # Type related data
    type = fields.Selection([
        ('media', 'Media'),
        ('custom_code', 'Custom Code'),
    ], 'Type', required=True)
    file = fields.Many2One(
        'nereid.static.file', 'File',
        states={
            'required': Equal(Eval('type'), 'media'),
            'invisible': Not(Equal(Eval('type'), 'media'))
        }
    )
    custom_code = fields.Text(
        'Custom Code', translate=True,
        states={
            'required': Equal(Eval('type'), 'custom_code'),
            'invisible': Not(Equal(Eval('type'), 'custom_code'))
        }
    )

    # Presentation related Data
    height = fields.Integer(
        'Height',
        states={
            'invisible': Not(Equal(Eval('type'), 'media'))
        }
    )
    width = fields.Integer(
        'Width',
        states={
            'invisible': Not(Equal(Eval('type'), 'media'))
        }
    )
    alternative_text = fields.Char(
        'Alternative Text', translate=True,
        states={
            'invisible': Not(Equal(Eval('type'), 'media'))
        }
    )
    click_url = fields.Char(
        'Click URL', translate=True,
        states={
            'invisible': Not(Equal(Eval('type'), 'media'))
        }
    )

    state = fields.Selection([
            ('draft', 'Draft'),
            ('published', 'Published'),
            ('archived', 'Archived')
    ], 'State', required=True, select=True, readonly=True)
    reference = fields.Reference('Reference', selection='allowed_models')

    @classmethod
    def view_attributes(cls):
        return super(Banner, cls).view_attributes() + [
            ('//page[@id="image"]', 'states', {
                'invisible': Eval('type') == 'custom_code'
            }), ('//page[@id="custom_code"]', 'states', {
                'invisible': Eval('type') != 'custom_code'
            })]

    @classmethod
    def __setup__(cls):
        super(Banner, cls).__setup__()
        cls._order.insert(0, ('sequence', 'ASC'))
        cls._transitions |= set((
                ('draft', 'published'),
                ('archived', 'published'),
                ('published', 'archived'),
        ))
        cls._buttons.update({
            'archive': {
                'invisible': Eval('state') != 'published',
            },
            'publish': {
                'invisible': Eval('state') == 'published',
            }
        })

    @classmethod
    @ModelView.button
    @Workflow.transition('archived')
    def archive(cls, banners):
        pass

    @classmethod
    @ModelView.button
    @Workflow.transition('published')
    def publish(cls, banners):
        pass

    def get_html(self):
        """Return the HTML content"""
        StaticFile = Pool().get('nereid.static.file')

        # XXX: Use read, as all fields are not required
        banner = self.read(
            [self], [
                'type', 'click_url', 'file', 'file.mimetype',
                'custom_code', 'height', 'width',
                'alternative_text', 'click_url'
            ]
        )[0]

        if banner['type'] == 'media':
            if not (banner['file.mimetype'] and
                    banner['file.mimetype'].startswith('image')):
                return "<!-- no html defined for mime type '%s' -->" % \
                    banner['file.mimetype']
            # replace the `file` in the dictionary with the complete url
            # that is required to render the image based on static file
            file = StaticFile(banner['file'])
            banner['file'] = file.url
            image = Template(
                u'<a href="$click_url">'
                u'<img src="$file" alt="$alternative_text"'
                u' width="$width" height="$height"/>'
                u'</a>'
            )
            return image.substitute(**banner)
        elif banner['type'] == 'custom_code':
            return banner['custom_code']

    @classmethod
    def allowed_models(cls):
        MenuItem = Pool().get('nereid.cms.menuitem')

        return MenuItem.allowed_models()

    @staticmethod
    def default_type():
        return 'media'

    @staticmethod
    def default_state():
        if 'published' in Transaction().context:
            return 'published'
        return 'draft'


class ArticleCategory(ModelSQL, ModelView, CMSMenuItemMixin):
    "Article Categories"
    __name__ = 'nereid.cms.article.category'
    _rec_name = 'title'

    title = fields.Char(
        'Title', size=100, translate=True, required=True, select=True
    )
    unique_name = fields.Char(
        'Unique Name', required=True, select=True,
        help='Unique Name is used as the uri.'
    )
    active = fields.Boolean('Active', select=True)
    description = fields.Text('Description', translate=True)
    template = fields.Char('Template', required=True)
    articles = fields.Many2Many(
        'nereid.cms.category-article', 'category', 'article', 'Article',
        context={'published': True}
    )

    # Article Category can have a banner
    banner = fields.Many2One('nereid.cms.banner', 'Banner')
    sort_order = fields.Selection([
        ('sequence', 'Sequence'),
        ('older_first', 'Older First'),
        ('recent_first', 'Recent First'),
    ], 'Sort Order')
    published_articles = fields.Function(
        fields.One2Many(
            'nereid.cms.article', 'category', 'Published Articles'
        ), 'get_published_articles'
    )
    articles_per_page = fields.Integer('Articles per Page', required=True)

    @staticmethod
    def default_sort_order():
        return 'recent_first'

    @staticmethod
    def default_active():
        'Return True'
        return True

    @staticmethod
    def default_template():
        return 'article-category.jinja'

    @classmethod
    def __setup__(cls):
        super(ArticleCategory, cls).__setup__()
        table = cls.__table__()
        cls._sql_constraints += [
            ('unique_name', Unique(table, table.unique_name),
                'The Unique Name of the Category must be unique.'),
        ]

    @fields.depends('title', 'unique_name')
    def on_change_title(self):
        if self.title and not self.unique_name:
            self.unique_name = slugify(self.title)

    @staticmethod
    def default_articles_per_page():
        return 10

    @classmethod
    @route('/article-category/<uri>/')
    @route('/article-category/<uri>/<int:page>')
    def render(cls, uri, page=1):
        """
        Renders the category
        """
        Article = Pool().get('nereid.cms.article')

        # Find in cache or load from DB
        try:
            category, = cls.search([('unique_name', '=', uri)])
        except ValueError:
            abort(404)

        order = []
        if category.sort_order == 'recent_first':
            order.append(('write_date', 'DESC'))
        elif category.sort_order == 'older_first':
            order.append(('write_date', 'ASC'))
        elif category.sort_order == 'sequence':
            order.append(('sequence', 'ASC'))

        articles = Pagination(
            Article, [
                ('categories', '=', category.id),
                ('state', '=', 'published')
            ], page, category.articles_per_page, order=order
        )
        return render_template(
            category.template, category=category, articles=articles)

    @classmethod
    @context_processor('get_article_category')
    def get_article_category(cls, uri, silent=True):
        """Returns the browse record of the article category given by uri
        """
        category = cls.search([('unique_name', '=', uri)], limit=1)
        if not category and not silent:
            raise RuntimeError("Article category %s not found" % uri)
        return category[0] if category else None

    @classmethod
    @route('/sitemaps/article-category-index.xml')
    def sitemap_index(cls):
        index = SitemapIndex(cls, [])
        return index.render()

    @classmethod
    @route('/sitemaps/article-category-<int:page>.xml')
    def sitemap(cls, page):
        sitemap_section = SitemapSection(cls, [], page)
        sitemap_section.changefreq = 'daily'
        return sitemap_section.render()

    def get_absolute_url(self, **kwargs):
        return url_for(
            'nereid.cms.article.category.render',
            uri=self.unique_name, **kwargs
        )

    def get_published_articles(self, name):
        """
        Get the published articles.
        """
        NereidArticle = Pool().get('nereid.cms.article')

        articles = NereidArticle.search([
            ('state', '=', 'published'),
            ('categories', '=', self.id)
        ])
        return map(int, articles)

    def get_children(self, max_depth):
        """
        Return serialized menu_item for current menu_item children
        """
        NereidArticle = Pool().get('nereid.cms.article')

        articles = NereidArticle.search([
            ('state', '=', 'published'),
            ('categories', '=', self.id)
        ])
        return [
            article.get_menu_item(max_depth=max_depth - 1)
            for article in articles
        ]

    def serialize(self, purpose=None):
        """
        Article category serialize method
        """
        if purpose == 'atom':
            return {
                'term': self.unique_name,
            }
        elif hasattr(super(ArticleCategory, self), 'serialize'):
            return super(ArticleCategory, self).serialize(purpose=purpose)

    @classmethod
    @route('/article-category/<uri>.atom')
    def atom_feed(cls, uri):
        """
        Returns atom feed for articles published under a particular category.
        """
        try:
            category, = cls.search([
                ('unique_name', '=', uri),
            ], limit=1)
        except ValueError:
            abort(404)

        feed = AtomFeed(
            "Articles by Category %s" % category.unique_name,
            feed_url=request.url, url=request.host_url
        )
        for article in category.published_articles:
            feed.add(**article.serialize(purpose='atom'))

        return feed.get_response()


class Article(Workflow, ModelSQL, ModelView, CMSMenuItemMixin):
    "CMS Articles"
    __name__ = 'nereid.cms.article'
    _rec_name = 'uri'

    uri = fields.Char('URI', required=True, select=True, translate=True)
    title = fields.Char('Title', required=True, select=True, translate=True)
    content = fields.Text('Content', required=True, translate=True)
    template = fields.Char('Template', required=True)
    image = fields.Many2One('nereid.static.file', 'Image')
    employee = fields.Many2One('company.employee', 'Employee')
    author = fields.Many2One('nereid.user', 'Author')
    published_on = fields.Date('Published On')
    publish_date = fields.Function(
        fields.Char('Publish Date'), 'get_publish_date'
    )
    sequence = fields.Integer('Sequence', required=True, select=True)
    reference = fields.Reference('Reference', selection='allowed_models')
    description = fields.Text('Short Description')
    attributes = fields.One2Many(
        'nereid.cms.article.attribute', 'article', 'Attributes'
    )
    categories = fields.Many2Many(
        'nereid.cms.category-article', 'article', 'category', 'Categories',
    )
    content_type = fields.Selection(
        'content_type_selection', 'Content Type',
        required=True
    )
    # Article can have a banner
    banner = fields.Many2One('nereid.cms.banner', 'Banner')
    state = fields.Selection([
        ('draft', 'Draft'),
        ('published', 'Published'),
        ('archived', 'Archived')
    ], 'State', required=True, select=True, readonly=True)

    @classmethod
    def __setup__(cls):
        super(Article, cls).__setup__()
        cls._order.insert(0, ('sequence', 'ASC'))
        cls._transitions |= set((
                ('draft', 'published'),
                ('published', 'draft'),
                ('published', 'archived'),
                ('archived', 'draft'),
        ))
        cls._buttons.update({
            'archive': {
                'invisible': Eval('state') != 'published',
            },
            'publish': {
                'invisible': Eval('state').in_(['published', 'archived']),
            },
            'draft': {
                'invisible': Eval('state') == 'draft',
            }
        })

    @classmethod
    def content_type_selection(cls):
        """
        Returns a selection for content_type.
        """
        default_types = [
            ('html', 'HTML'),
            ('plain', 'Plain Text')
        ]

        if markdown:
            default_types.append(('markdown', 'Markdown'))
        if publish_parts:
            default_types.append(('rst', 'reStructured TeXT'))

        return default_types

    @classmethod
    def default_content_type(cls):
        """
        Default content_type.
        """
        return 'plain'

    def __html__(self):
        """
        Uses content_type field to generate html content.
        Concept from Jinja2's Markup class.
        """
        if self.content_type == 'rst':
            if publish_parts:
                res = publish_parts(self.content, writer_name='html')
                return res['html_body']
            self.raise_user_error(
                "`docutils` not installed, to render rst articles."
            )
        if self.content_type == 'markdown':
            if markdown:
                return markdown(self.content)
            self.raise_user_error(
                "`markdown` not installed, to render markdown article."
            )
        return self.content

    @classmethod
    @ModelView.button
    @Workflow.transition('archived')
    def archive(cls, articles):
        pass

    @classmethod
    @ModelView.button
    @Workflow.transition('published')
    def publish(cls, articles):
        pass

    @classmethod
    @ModelView.button
    @Workflow.transition('draft')
    def draft(cls, articles):
        pass

    @classmethod
    def allowed_models(cls):
        MenuItem = Pool().get('nereid.cms.menuitem')

        return MenuItem.allowed_models()

    @fields.depends('title', 'uri')
    def on_change_title(self):
        if self.title and not self.uri:
            self.uri = slugify(self.title)

    @staticmethod
    def default_template():
        return 'article.jinja'

    @staticmethod
    def default_employee():
        User = Pool().get('res.user')

        if 'employee' in Transaction().context:
            return Transaction().context['employee']

        user = User(Transaction().user)
        if user.employee:
            return user.employee.id

        if has_request_context() and current_user.employee:
            return current_user.employee.id

    @staticmethod
    def default_author():
        if has_request_context():
            return current_user.id

    @staticmethod
    def default_published_on():
        Date = Pool().get('ir.date')
        return Date.today()

    @classmethod
    @route('/article/<uri>')
    def render(cls, uri):
        """
        Renders the template
        """
        try:
            article, = cls.search([
                ('uri', '=', uri),
                ('state', '=', 'published'),
            ])
        except ValueError:
            abort(404)
        return render_template(article.template, article=article)

    @classmethod
    @route('/sitemaps/article-index.xml')
    def sitemap_index(cls):
        index = SitemapIndex(cls, [])
        return index.render()

    @classmethod
    @route('/sitemaps/article-<int:page>.xml')
    def sitemap(cls, page):
        sitemap_section = SitemapSection(cls, [], page)
        sitemap_section.changefreq = 'daily'
        return sitemap_section.render()

    @classmethod
    def get_publish_date(cls, records, name):
        """
        Return publish date to render on view
        """
        res = {}
        for record in records:
            res[record.id] = str(record.published_on)
        return res

    def get_absolute_url(self, **kwargs):
        return url_for(
            'nereid.cms.article.render', uri=self.uri, **kwargs
        )

    @staticmethod
    def default_state():
        if 'published' in Transaction().context:
            return 'published'
        return 'draft'

    def get_menu_item(self, max_depth):
        """
        Return huge dictionary with serialized article category for menu item

        {
            title: <display name>,
            link: <url>,
            record: <instance of record>  # if type_ is `record`
        }
        """
        return {
            'record': self,
            'title': self.title,
            'link': has_request_context() and self.get_absolute_url(),
        }

    def atom_id(self):
        """
        Returns an atom ID for the article
        """
        return (
            'tag:' + current_website.name + ',' +
            self.publish_date + ':Article/' + str(self.id)
        )

    def atom_publish_date(self):
        """
        Returns the article's publish date with timezone set as UTC
        """
        return pytz.utc.localize(
            datetime.combine(self.published_on, datetime.min.time())
        )

    def serialize(self, purpose=None):
        """
        Serialize Article records
        """
        if purpose == 'atom':
            # The keys in the dictionary returned are used by Werkzeug's
            # AtomFeed class.
            return {
                'id': self.atom_id(),
                'title': self.title,
                'author': (
                    self.author.serialize(purpose=purpose) if self.author
                    else None
                ),
                'content': self.content,
                'content_type': (
                    'text' if self.content_type == 'plain' else 'html'
                ),
                'link': {
                    'rel': 'alternate',
                    'type': 'text/html',
                    'href': has_request_context() and self.get_absolute_url(external=True),
                },
                'category': [
                    category.serialize(purpose=purpose)
                    for category in self.categories
                ],
                'published': self.atom_publish_date(),
                'updated': self.write_date or self.atom_publish_date(),
            }
        elif hasattr(super(Article, self), 'serialize'):
            return super(Article, self).serialize(purpose=purpose)

    @classmethod
    @route('/article/all.atom')
    def atom_feed(cls):
        """
        Renders the atom feed for all articles.
        """
        feed = AtomFeed(
            "All Articles", feed_url=request.url, url=request.host_url
        )
        for article in cls.search([
            ('state', '=', 'published')
        ]):
            feed.add(**article.serialize(purpose='atom'))

        return feed.get_response()


class ArticleAttribute(ModelSQL, ModelView):
    "Articles Attribute"
    __name__ = 'nereid.cms.article.attribute'
    _rec_name = 'value'

    name = fields.Selection([
        ('', None),
        ('google+', 'Google+'),
        ('facebook', 'Facebook'),
        ('twitter', 'Twitter'),
        ('github', 'Github'),
        ('linked-in', 'Linked-in'),
        ('blogger', 'Blogger'),
        ('tumblr', 'Tumblr'),
        ('website', 'Website'),
        ('phone', 'Phone'),
    ], 'Name', required=True, select=True)
    value = fields.Char('Value', required=True)
    article = fields.Many2One(
        'nereid.cms.article', 'Article', ondelete='CASCADE', required=True,
        select=True,
    )


class NereidStaticFile:
    "Nereid Static File"
    __name__ = 'nereid.static.file'

    def serialize(self):
        """
        Serialize this object
        """
        return {
            'name': self.name,
            'get_url': self.url,
        }


class Website:
    "Nereid Website"
    __name__ = 'nereid.website'

    cms_static_folder = fields.Many2One(
        'nereid.static.folder', "CMS Static Folder", ondelete='RESTRICT',
        select=True,
    )

    @classmethod
    @route('/cms/upload/<upload_type>', methods=['POST'])
    @login_required
    def cms_static_upload(cls, upload_type):
        """
        Upload the file for cms
        """
        StaticFile = Pool().get("nereid.static.file")

        file = request.files['file']
        if file:
            static_file, = StaticFile.create([{
                'folder': current_website.cms_static_folder,
                'name': '_'.join([
                    str(int(time.time())),
                    secure_filename(file.filename),
                ]),
                'type': upload_type,
                'file_binary': file.read(),
            }])
            if request.is_xhr:
                return jsonify(success=True, item=static_file.serialize())

            flash("File uploaded")
        if request.is_xhr:
            return jsonify(success=False)
        return redirect(request.referrer)

    @classmethod
    @route('/cms/browse', methods=['POST'])
    @route('/cms/browse/<int:page>', methods=['GET'])
    @login_required
    def cms_static_list(cls, page=1):
        """
            Return JSON with list of all files inside cms static folder
        """
        StaticFile = Pool().get("nereid.static.file")

        files = Pagination(
            StaticFile, [
                ('folder', '=', current_website.cms_static_folder.id)
            ], page, 10
        )
        return jsonify(items=[
            item.serialize() for item in files
        ])


class ArticleCategoryRelation(ModelSQL):
    """
    Relationshiop between article and category
    """
    __name__ = 'nereid.cms.category-article'

    category = fields.Many2One(
        'nereid.cms.article.category', 'Category', select=True
    )
    article = fields.Many2One(
        'nereid.cms.article', 'Article', select=True
    )



#!/usr/bin/env python
# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
import unittest
import sys
from setuptools import setup, Command
import re
import os
import ConfigParser


class SQLiteTest(Command):
    """
    Run the tests on SQLite
    """
    description = "Run tests on SQLite"

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.environ['TRYTOND_DATABASE_URI'] = 'sqlite://'
        os.environ['DB_NAME'] = ':memory:'

        from tests import suite
        test_result = unittest.TextTestRunner(verbosity=3).run(suite())

        if test_result.wasSuccessful():
            sys.exit(0)
        sys.exit(-1)


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


config = ConfigParser.ConfigParser()
config.readfp(open('tryton.cfg'))
info = dict(config.items('tryton'))
for key in ('depends', 'extras_depend', 'xml'):
    if key in info:
        info[key] = info[key].strip().splitlines()
major_version, minor_version, _ = info.get('version', '0.0.1').split('.', 2)
major_version = int(major_version)
minor_version = int(minor_version)

requires = []
for dep in info.get('depends', []):
    if not re.match(r'(ir|res|webdav)(\W|$)', dep):
        requires.append(
            'trytond_%s >= %s.%s, < %s.%s' % (
                dep, major_version, minor_version, major_version,
                minor_version + 1
            )
        )
requires.append(
    'trytond >= %s.%s, < %s.%s' % (
        major_version, minor_version, major_version, minor_version + 1
    )
)

test_requirements = ['docutils', 'markdown']

setup(
    name='fio_nereid_cms',
    version=info.get('version', '0.0.1'),
    description='Tryton Nereid CMS',
    author='Fulfil.IO Inc., Openlabs Technologies & Consulting (P) Limited',
    author_email='info@fulfil.io',
    url='http://www.fulfil.io/',
    package_dir={'trytond.modules.nereid_cms': '.'},
    packages=[
        'trytond.modules.nereid_cms',
        'trytond.modules.nereid_cms.tests',
    ],
    package_data={
        'trytond.modules.nereid_cms': info.get('xml', []) + [
            'tryton.cfg', 'locale/*.po', '*.odt', 'icons/*.svg', 'view/*.xml',
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Plugins',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Tryton',
        'Topic :: Office/Business',
    ],
    license='BSD',
    install_requires=requires,
    tests_require=test_requirements,
    zip_safe=False,
    entry_points="""
    [trytond.modules]
    nereid_cms = trytond.modules.nereid_cms
    """,
    test_suite='tests',
    test_loader='trytond.test_loader:Loader',
    cmdclass={
        'test': SQLiteTest,
    },
)
